@@ -649,7 +649,7 @@ int set_auth_mode(struct wrap_ndis_device *wnd, ULONG auth_mode)
 	TRACEENTER2("%d", auth_mode);
 	res = miniport_set_int(wnd, OID_802_11_AUTHENTICATION_MODE, auth_mode);
 	if (res) {
-		WARNING("setting auth mode to %d failed (%08X)",
+		WARNING("setting auth mode to %u failed (%08X)",
 			auth_mode, res);
 		if (res == NDIS_STATUS_INVALID_DATA)
 			TRACEEXIT2(return -EINVAL);
@@ -680,7 +680,8 @@ int set_encr_mode(struct wrap_ndis_device *wnd, ULONG encr_mode)
 	TRACEENTER2("%d", encr_mode);
 	res = miniport_set_int(wnd, OID_802_11_ENCRYPTION_STATUS, encr_mode);
 	if (res) {
-		WARNING("setting encryption status failed (%08X)", res);
+		WARNING("setting encryption mode to %u failed (%08X)",
+			encr_mode, res);
 		if (res == NDIS_STATUS_INVALID_DATA)
 			TRACEEXIT2(return -EINVAL);
 		return -EOPNOTSUPP;
@@ -1092,7 +1093,7 @@ static char *ndis_translate_scan(struct net_device *dev, char *event,
 	iwe.u.data.length = strlen(buf);
 	event = iwe_stream_add_point(event, end_buf, &iwe, buf);
 
-	DBGTRACE2("%d, %u", bssid->length, sizeof(*bssid));
+	DBGTRACE2("%d, %u", bssid->length, (unsigned int)sizeof(*bssid));
 	if (bssid->length > sizeof(*bssid)) {
 		unsigned char *iep = (unsigned char *)bssid_ex->ies +
 			sizeof(struct ndis_fixed_ies);
@@ -1849,9 +1850,9 @@ static int priv_power_profile(struct net_device *dev,
 	else
 		profile_inf = NdisPowerProfileBattery;
 
-	miniport->pnp_event_notify(wnd->nmb->adapter_ctx,
-				   NdisDevicePnPEventPowerProfileChanged,
-				   &profile_inf, sizeof(profile_inf));
+	LIN2WIN4(miniport->pnp_event_notify, wnd->nmb->adapter_ctx,
+		 NdisDevicePnPEventPowerProfileChanged,
+		 &profile_inf, sizeof(profile_inf));
 	TRACEEXIT2(return 0);
 }
 
