@@ -13,24 +13,25 @@
  *
  */
 
-#ifndef LOADER_H
-#define LOADER_H
+#ifndef _LOADER_H_
+#define _LOADER_H_
 
 #include "ndiswrapper.h"
 
 struct load_driver_file {
+	char driver_name[MAX_DRIVER_NAME_LEN];
 	char name[MAX_DRIVER_NAME_LEN];
 	size_t size;
 	void *data;
 };
 
 struct load_device_setting {
-	char name[MAX_NDIS_SETTING_NAME_LEN];
-	char value[MAX_NDIS_SETTING_VALUE_LEN];
+	char name[MAX_SETTING_NAME_LEN];
+	char value[MAX_SETTING_VALUE_LEN];
 };
-		
+
 struct load_device {
-	int bustype;
+	int bus_type;
 	int vendor;
 	int device;
 	int subvendor;
@@ -48,20 +49,33 @@ struct load_driver {
 	char name[MAX_DRIVER_NAME_LEN];
 	char conf_file_name[MAX_DRIVER_NAME_LEN];
 	unsigned int nr_sys_files;
-	struct load_driver_file sys_files[MAX_PE_IMAGES];
+	struct load_driver_file sys_files[MAX_DRIVER_PE_IMAGES];
 	unsigned int nr_settings;
-	struct load_device_setting settings[MAX_NDIS_SETTINGS];
+	struct load_device_setting settings[MAX_DEVICE_SETTINGS];
 	unsigned int nr_bin_files;
-	struct load_driver_file bin_files[MAX_NDIS_BIN_FILES];
+	struct load_driver_file bin_files[MAX_DRIVER_BIN_FILES];
 };
 
-#define NDIS_REGISTER_DEVICES	_IOW(('N' + 'd' + 'i' + 'S'), 0,	\
-				     struct load_devices *)
-#define NDIS_LOAD_DRIVER	_IOW(('N' + 'd' + 'i' + 'S'), 1,	\
-				     struct load_driver *)
+#define WRAP_IOCTL_REGISTER_DEVICES _IOW(('N' + 'd' + 'i' + 'S'), 0,	\
+					 struct load_devices *)
+#define WRAP_IOCTL_LOAD_DRIVER _IOW(('N' + 'd' + 'i' + 'S'), 1,	\
+				    struct load_driver *)
+#define WRAP_IOCTL_LOAD_BIN_FILE _IOW(('N' + 'd' + 'i' + 'S'), 2,	\
+				      struct load_driver_file *)
+
+#define WRAP_CMD_LOAD_DEVICES "load_devices"
+#define WRAP_CMD_LOAD_DRIVER "load_driver"
+#define WRAP_CMD_LOAD_BIN_FILE "load_bin_file"
 
 int loader_init(void);
 void loader_exit(void);
 
+#ifdef __KERNEL__
+struct wrap_driver *load_wrap_driver(struct wrap_device *device);
+struct wrap_bin_file *get_bin_file(char *bin_file_name);
+void free_bin_file(struct wrap_bin_file *bin_file);
+void unload_wrap_driver(struct wrap_driver *driver);
+#endif
+
 #endif /* LOADER_H */
 
