@@ -1319,7 +1319,7 @@ wstdcall void WRAP_EXPORT(NdisMCompleteBufferPhysicalMapping)
 		      map_index, wnd->dma_map_count);
 		return;
 	}
-	DBGTRACE4("%x, %d, %d", wnd->dma_map_addr[map_index],
+	DBGTRACE4("%lx, %d, %d", (unsigned long)wnd->dma_map_addr[map_index],
 		  MmGetMdlByteCount(buf), map_index);
 	if (wnd->dma_map_addr[map_index] == 0) {
 //		ERROR("map register not used (%lu)", map_index);
@@ -1773,7 +1773,7 @@ wstdcall void WRAP_EXPORT(NdisMDeregisterAdapterShutdownHandler)
 	wnd->shutdown_ctx = NULL;
 }
 
-static void ndis_irq_handler(unsigned long data)
+static void ndis_irq_handler_dynamic(unsigned long data)
 {
 	struct wrap_ndis_device *wnd = (struct wrap_ndis_device *)data;
 	struct miniport_char *miniport;
@@ -1784,33 +1784,52 @@ static void ndis_irq_handler(unsigned long data)
 		LIN2WIN1(miniport->enable_interrupts, wnd->nmb->adapter_ctx);
 }
 
-irqreturn_t ndis_isr(int irq, void *data, struct pt_regs *pt_regs)
+irqreturn_t ndis_isr_dynamic(int irq, void *data, struct pt_regs *pt_regs)
 {
 	struct wrap_ndis_device *wnd = (struct wrap_ndis_device *)data;
-	int recognized, queue_handler;
 	struct miniport_char *miniport;
 
 	miniport = &wnd->wd->driver->ndis_driver->miniport;
 	/* this spinlock should be shared with NdisMSynchronizeWithInterrupt
 	 */
 	nt_spin_lock(&wnd->ndis_irq->lock);
-	recognized = queue_handler = 0;
-	if (wnd->ndis_irq->req_isr)
-		LIN2WIN3(miniport->isr, &recognized, &queue_handler,
-			 wnd->nmb->adapter_ctx);
-	else { //if (miniport->disable_interrupts)
-		LIN2WIN1(miniport->disable_interrupts, wnd->nmb->adapter_ctx);
-		/* it is not shared interrupt, so handler must be called */
-		recognized = queue_handler = 1;
-	}
+	LIN2WIN1(miniport->disable_interrupts, wnd->nmb->adapter_ctx);
+	nt_spin_unlock(&wnd->ndis_irq->lock);
+
+	/* it is not shared interrupt, so handler must be called */
+	tasklet_schedule(&wnd->irq_tasklet);
+	return IRQ_HANDLED;
+}
+
+static void ndis_irq_handler_shared(unsigned long data)
+{
+	struct wrap_ndis_device *wnd = (struct wrap_ndis_device *)data;
+	struct miniport_char *miniport;
+
+	miniport = &wnd->wd->driver->ndis_driver->miniport;
+	LIN2WIN1(miniport->handle_interrupt, wnd->nmb->adapter_ctx);
+}
+
+irqreturn_t ndis_isr_shared(int irq, void *data, struct pt_regs *pt_regs)
+{
+	struct wrap_ndis_device *wnd = (struct wrap_ndis_device *)data;
+	BOOLEAN recognized, queue_handler;
+	struct miniport_char *miniport;
+
+	miniport = &wnd->wd->driver->ndis_driver->miniport;
+	/* this spinlock should be shared with NdisMSynchronizeWithInterrupt
+	 */
+	nt_spin_lock(&wnd->ndis_irq->lock);
+	LIN2WIN3(miniport->isr, &recognized, &queue_handler,
+		 wnd->nmb->adapter_ctx);
 	nt_spin_unlock(&wnd->ndis_irq->lock);
 
 	if (recognized) {
 		if (queue_handler)
 			tasklet_schedule(&wnd->irq_tasklet);
 		return IRQ_HANDLED;
-	}
-	return IRQ_NONE;
+	} else
+		return IRQ_NONE;
 }
 
 wstdcall NDIS_STATUS WRAP_EXPORT(NdisMRegisterInterrupt)
@@ -1831,9 +1850,14 @@ wstdcall NDIS_STATUS WRAP_EXPORT(NdisMRegisterInterrupt)
 	nt_spin_lock_init(&ndis_irq->lock);
 	wnd->ndis_irq = ndis_irq;
 
-	tasklet_init(&wnd->irq_tasklet, ndis_irq_handler, (unsigned long)wnd);
-	if (request_irq(vector, ndis_isr, req_isr ? SA_SHIRQ : 0,
-			"ndiswrapper", wnd)) {
+	if (req_isr)
+		tasklet_init(&wnd->irq_tasklet, ndis_irq_handler_shared,
+			     (unsigned long)wnd);
+	else
+		tasklet_init(&wnd->irq_tasklet, ndis_irq_handler_dynamic,
+			     (unsigned long)wnd);
+	if (request_irq(vector, req_isr ? ndis_isr_shared : ndis_isr_dynamic,
+			req_isr ? SA_SHIRQ : 0, "ndiswrapper", wnd)) {
 		printk(KERN_WARNING "%s: request for irq %d failed\n",
 		       DRIVER_NAME, vector);
 		TRACEEXIT1(return NDIS_STATUS_RESOURCES);
@@ -2136,16 +2160,33 @@ NdisMSendComplete(struct ndis_miniport_block *nmb, struct ndis_packet *packet,
 		  NDIS_STATUS status)
 {
 	struct wrap_ndis_device *wnd = nmb->wnd;
-	TRACEENTER3("%p, %08x", packet, status);
-	free_tx_packet(wnd, packet, status);
-	/* In case a serialized driver has earlier requested a pause
-	 * by returning NDIS_STATUS_RESOURCES during
-	 * MiniportSend(Packets), wakeup tx worker now.
-	 */
-	if (wnd->tx_ok == 0) {
-		wnd->tx_ok = 1;
-		DBGTRACE3("%d, %d", wnd->tx_ring_start, wnd->tx_ring_end);
-		schedule_wrap_work(&wnd->tx_work);
+	if (wnd->attributes & NDIS_ATTRIBUTE_DESERIALIZE)
+		free_tx_packet(wnd, packet, status);
+	else {
+		struct ndis_packet_oob_data *oob_data;
+
+		TRACEENTER3("%p, %08x", packet, status);
+		oob_data = NDIS_PACKET_OOB_DATA(packet);
+		switch (xchg(&oob_data->status, status)) {
+		case NDIS_STATUS_PENDING:
+			break;
+		case NDIS_STATUS_NOT_RECOGNIZED:
+			free_tx_packet(wnd, packet, status);
+			break;
+		default:
+			WARNING("invalid status");
+			break;
+		}
+		/* In case a serialized driver has earlier requested a
+		 * pause by returning NDIS_STATUS_RESOURCES during
+		 * MiniportSend(Packets), wakeup tx worker now.
+		 */
+		if (wnd->tx_ok == 0) {
+			wnd->tx_ok = 1;
+			DBGTRACE3("%d, %d", wnd->tx_ring_start,
+				  wnd->tx_ring_end);
+			schedule_wrap_work(&wnd->tx_work);
+		}
 	}
 	TRACEEXIT3(return);
 }
