@@ -340,8 +340,8 @@ struct ndis_pnp_capabilities {
 	struct ndis_pm_wakeup_capabilities wakeup_capa;
 };
 
-typedef void (*ndis_isr_handler)(unsigned int *taken, unsigned int *callme,
-				 void *ctx) wstdcall;
+typedef void (*ndis_isr_handler)(BOOLEAN *recognized, BOOLEAN *queue_handler,
+				 void *handle) wstdcall;
 typedef void (*ndis_interrupt_handler)(void *ctx) wstdcall;
 
 struct miniport_char {
@@ -901,7 +901,8 @@ wstdcall void NdisMResetComplete(struct ndis_miniport_block *nmb,
 wstdcall ULONG NDIS_BUFFER_TO_SPAN_PAGES(ndis_buffer *buffer);
 wstdcall BOOLEAN NdisWaitEvent(struct ndis_event *event, UINT timeout);
 wstdcall void NdisSetEvent(struct ndis_event *event);
-irqreturn_t ndis_isr(int irq, void *data, struct pt_regs *pt_regs);
+irqreturn_t ndis_isr_shared(int irq, void *data, struct pt_regs *pt_regs);
+irqreturn_t ndis_isr_dynamic(int irq, void *data, struct pt_regs *pt_regs);
 wstdcall void NdisMDeregisterInterrupt(struct ndis_irq *ndis_irq);
 wstdcall void EthRxIndicateHandler
 	(struct ndis_miniport_block *nmb, void *rx_ctx, char *header1,
