@@ -969,8 +969,7 @@ static void ntos_work_item_worker(void *data)
 		nt_spin_unlock_irql(&ntos_work_item_list_lock, irql);
 		if (!cur)
 			break;
-		ntos_work_item = container_of(cur, struct ntos_work_item,
-					      list);
+		ntos_work_item = container_of(cur, struct ntos_work_item, list);
 		func = ntos_work_item->func;
 		WORKTRACE("%p: executing %p, %p, %p", current, func,
 			  ntos_work_item->arg1, ntos_work_item->arg2);
@@ -1179,7 +1178,7 @@ wstdcall void WRAP_EXPORT(ExDeleteNPagedLookasideList)
 	TRACEEXIT3(return);
 }
 
-#ifdef ALLOC_DEBUG
+#if defined(ALLOC_DEBUG) && ALLOC_DEBUG > 1
 #define ExAllocatePoolWithTag(pool_type, size, tag)			\
 	wrap_ExAllocatePoolWithTag(pool_type, size, tag, __FILE__, __LINE__)
 #endif
@@ -1625,7 +1624,7 @@ wstdcall void WRAP_EXPORT(KeClearEvent)
 	(struct nt_event *nt_event)
 {
 	EVENTENTER("event = %p", nt_event);
-	xchg(&nt_event->dh.signal_state, 0);
+	(void)xchg(&nt_event->dh.signal_state, 0);
 	EVENTEXIT(return);
 }
 
@@ -1879,7 +1878,6 @@ static struct nt_thread *create_nt_thread(struct task_struct *task)
 		initialize_dh(&thread->dh, ThreadObject, 0);
 		thread->dh.size = sizeof(*thread);
 		nt_spin_unlock_irql(&dispatcher_lock, irql);
-
 		DBGTRACE2("thread: %p, task: %p, pid: %d",
 			  thread, thread->task, thread->pid);
 	} else
