@@ -159,9 +159,8 @@ wstdcall BOOLEAN WRAP_EXPORT(IoCancelIrp)
 		return FALSE;
 	DUMP_IRP(irp);
 	IoAcquireCancelSpinLock(&irp->cancel_irql);
-	cancel_routine = irp->cancel_routine;
+	cancel_routine = xchg(&irp->cancel_routine, NULL);
 	IOTRACE("%p", cancel_routine);
-	irp->cancel_routine = NULL;
 	irp->cancel = TRUE;
 	if (cancel_routine) {
 		struct io_stack_location *irp_sl;
@@ -169,9 +168,15 @@ wstdcall BOOLEAN WRAP_EXPORT(IoCancelIrp)
 		IOTRACE("%p, %p", irp_sl, irp_sl->dev_obj);
 		/* cancel_routine will release the spin lock */
 		LIN2WIN2(cancel_routine, irp_sl->dev_obj, irp);
-		IOEXIT(return TRUE);
+		/* in usb's cancel, irp->cancel is set to indicate
+		 * status of cancel */
+		if (irp->cancel == FALSE) {
+			irp->cancel = TRUE;
+			IOEXIT(return FALSE);
+		} else
+			IOEXIT(return TRUE);
 	} else {
-		IOTRACE("irp %p not canceled", irp);
+		IOTRACE("irp %p already canceled", irp);
 		IoReleaseCancelSpinLock(irp->cancel_irql);
 		IOEXIT(return FALSE);
 	}
