@@ -84,13 +84,9 @@ static int inline wrap_cancel_urb(struct wrap_urb *wrap_urb)
 	if (wrap_urb->state != URB_SUBMITTED)
 		USBEXIT(return -1);
 	ret = usb_unlink_urb(wrap_urb->urb);
-	if (ret == -EINPROGRESS) {
-		if (wrap_urb->state != URB_SUBMITTED)
-			WARNING("urb %p in wrong state: %d",
-				wrap_urb->urb, wrap_urb->state);
-		wrap_urb->state = URB_CANCELED;
+	if (ret == -EINPROGRESS)
 		return 0;
-	} else {
+	else {
 		WARNING("unlink failed: %d", ret);
 		return ret;
 	}
@@ -132,7 +128,7 @@ static void usb_kill_urb(struct urb *urb)
 static struct nt_list wrap_urb_complete_list;
 static NT_SPIN_LOCK wrap_urb_complete_list_lock;
 static wstdcall void wrap_cancel_irp(struct device_object *dev_obj,
-				    struct irp *irp);
+				     struct irp *irp);
 
 /* use tasklet instead worker to process completed urbs */
 #define USB_TASKLET 1
@@ -276,8 +272,7 @@ static void wrap_free_urb(struct urb *urb)
 	if (wrap_urb->flags & WRAP_URB_COPY_BUFFER) {
 		USBTRACE("freeing DMA buffer for URB: %p %p",
 			 urb, urb->transfer_buffer);
-		usb_buffer_free(irp->wd->usb.udev,
-				urb->transfer_buffer_length,
+		usb_buffer_free(irp->wd->usb.udev, urb->transfer_buffer_length,
 				urb->transfer_buffer, urb->transfer_dma);
 	}
 	if (urb->setup_packet)
@@ -482,8 +477,7 @@ static void wrap_urb_complete(struct urb *urb, struct pt_regs *regs)
 	DUMP_WRAP_URB(wrap_urb, USB_DIR_IN);
 	irp->cancel_routine = NULL;
 #ifdef USB_DEBUG
-	if (wrap_urb->state != URB_SUBMITTED &&
-	    wrap_urb->state != URB_CANCELED) {
+	if (wrap_urb->state != URB_SUBMITTED) {
 		WARNING("urb %p in wrong state: %d (%d)", urb, wrap_urb->state,
 			urb->status);
 		return;
@@ -574,22 +568,29 @@ static void wrap_urb_complete_worker(void *dummy)
 					       urb->transfer_buffer,
 					       urb->actual_length);
 			}
+			NT_URB_STATUS(nt_urb) =
+				wrap_urb_status(URB_STATUS(wrap_urb));
+			irp->io_status.status =
+				nt_urb_irp_status(NT_URB_STATUS(nt_urb));
 			break;
 		case -ENOENT:
 		case -ECONNRESET:
 			/* urb canceled */
 			irp->io_status.info = 0;
 			USBTRACE("urb %p canceled", urb);
+			NT_URB_STATUS(nt_urb) = USBD_STATUS_SUCCESS;
+			irp->io_status.status = STATUS_CANCELLED;
 			break;
 		default:
 			USBTRACE("irp: %p, urb: %p, status: %d",
 				 irp, urb, URB_STATUS(wrap_urb));
 			irp->io_status.info = 0;
+			NT_URB_STATUS(nt_urb) =
+				wrap_urb_status(URB_STATUS(wrap_urb));
+			irp->io_status.status =
+				nt_urb_irp_status(NT_URB_STATUS(nt_urb));
 			break;
 		}
-		NT_URB_STATUS(nt_urb) = wrap_urb_status(URB_STATUS(wrap_urb));
-		irp->io_status.status =
-			nt_urb_irp_status(NT_URB_STATUS(nt_urb));
 		wrap_free_urb(urb);
 		IoCompleteRequest(irp, IO_NO_INCREMENT);
 	}
@@ -599,7 +600,7 @@ static void wrap_urb_complete_worker(void *dummy)
 /* called as Windows function, so call WIN2LIN2 before accessing
  * arguments */
 static wstdcall void wrap_cancel_irp(struct device_object *dev_obj,
-				    struct irp *irp)
+				     struct irp *irp)
 {
 	struct urb *urb;
 
@@ -609,12 +610,12 @@ static wstdcall void wrap_cancel_irp(struct device_object *dev_obj,
 	USBENTER("irp: %p", irp);
 	urb = irp->wrap_urb->urb;
 	USBTRACE("canceling urb %p", urb);
-	if (wrap_cancel_urb(irp->wrap_urb) == 0) {
-		USBTRACE("urb %p canceled", urb);
-		/* this IRP will be returned in urb's completion function */
-	} else
-		ERROR("urb %p can't be canceld: %d", urb, irp->wrap_urb->state);
 	IoReleaseCancelSpinLock(irp->cancel_irql);
+	if (wrap_cancel_urb(irp->wrap_urb)) {
+		irp->cancel = FALSE;
+		ERROR("urb %p can't be canceld: %d", urb, irp->wrap_urb->state);
+	} else
+		USBTRACE("urb %p canceled", urb);
 	return;
 }
 
@@ -1281,8 +1282,7 @@ wstdcall union nt_urb *WRAP_EXPORT(USBD_CreateConfigurationRequestEx)
 	intf = &select_conf->intf;
 	/* handle points to beginning of interface information */
 	select_conf->handle = intf;
-	for (n = 0; n < config->bNumInterfaces && intf_list[n].intf_desc;
-	     n++) {
+	for (n = 0; n < config->bNumInterfaces && intf_list[n].intf_desc; n++) {
 		/* initialize 'intf' fields in intf_list so they point
 		 * to appropriate entry; these may be read/written by
 		 * driver after this function returns */
