@@ -106,7 +106,7 @@ typedef LONG USBD_STATUS;
 
 #define USBD_STATUS_SUCCESS			0x0
 #define USBD_STATUS_PENDING			0x40000000
-#define USBD_STATUS_CANCELLED			0x00010000
+#define USBD_STATUS_CANCELED			0x00010000
 
 #define USBD_STATUS_CRC				0xC0000001
 #define USBD_STATUS_BTSTUFF			0xC0000002
