@@ -480,6 +480,7 @@ static struct ndis_packet *alloc_tx_packet(struct wrap_ndis_device *wnd,
 	DBG_BLOCK(4) {
 		dump_bytes(__FUNCTION__, skb->data, skb->len);
 	}
+	DBGTRACE4("packet: %p, buffer: %p, skb: %p", packet, buffer, skb);
 	return packet;
 }
 
@@ -550,24 +551,28 @@ static int miniport_tx_packets(struct wrap_ndis_device *wnd)
 			int j = (start + i) % TX_RING_SIZE;
 			wnd->tx_array[i] = wnd->tx_ring[j];
 		}
-		if (!(wnd->attributes & NDIS_ATTRIBUTE_DESERIALIZE)) {
+		if (wnd->attributes & NDIS_ATTRIBUTE_DESERIALIZE) {
+			LIN2WIN3(miniport->send_packets, wnd->nmb->adapter_ctx,
+				 wnd->tx_array, n);
+			sent = n;
+		} else {
+			struct ndis_packet_oob_data *oob_data;
 			irql = raise_irql(DISPATCH_LEVEL);
 			LIN2WIN3(miniport->send_packets, wnd->nmb->adapter_ctx,
 				 wnd->tx_array, n);
 			lower_irql(irql);
 			for (sent = 0; sent < n && wnd->tx_ok; sent++) {
-				struct ndis_packet_oob_data *oob_data;
 				packet = wnd->tx_array[sent];
 				oob_data = NDIS_PACKET_OOB_DATA(packet);
-				switch (oob_data->status) {
+				switch (xchg(&oob_data->status,
+					     NDIS_STATUS_NOT_RECOGNIZED)) {
 				case NDIS_STATUS_SUCCESS:
 					free_tx_packet(wnd, packet,
-						       oob_data->status);
+						       NDIS_STATUS_SUCCESS);
 					break;
 				case NDIS_STATUS_PENDING:
 					break;
 				case NDIS_STATUS_RESOURCES:
-					DBGTRACE3("%d, %d", start, end);
 					wnd->tx_ok = 0;
 					/* resubmit this packet and
 					 * the rest when resources
@@ -575,49 +580,54 @@ static int miniport_tx_packets(struct wrap_ndis_device *wnd)
 					sent--;
 					break;
 				case NDIS_STATUS_FAILURE:
-					WARNING("packet %p dropped", packet);
+					free_tx_packet(wnd, packet,
+						       NDIS_STATUS_FAILURE);
+					break;
+				default:
+					ERROR("packet %p: invalid status",
+					      packet);
 					free_tx_packet(wnd, packet,
 						       oob_data->status);
 					break;
-				default:
-					ERROR("packet %p: unknown status %08X",
-					      packet, oob_data->status);
-					/* RT2500 driver seems to
-					 * corrupt packets with SMP
-					 * and by the time control
-					 * comes here, the buffer and
-					 * packet are already freed */
-					break;
 				}
 			}
-		} else {
-			LIN2WIN3(miniport->send_packets, wnd->nmb->adapter_ctx,
-				 wnd->tx_array, n);
-			sent = n;
 		}
 		DBGTRACE3("sent: %d(%d)", sent, n);
 	} else {
 		int i;
 		irql = PASSIVE_LEVEL;
-		if (!(wnd->attributes & NDIS_ATTRIBUTE_DESERIALIZE))
-			irql = raise_irql(DISPATCH_LEVEL);
-		for (i = 0; i < n; i++) {
+		for (i = 0; i < n && wnd->tx_ok; i++) {
+			struct ndis_packet_oob_data *oob_data;
 			packet = wnd->tx_ring[(start + i) % TX_RING_SIZE];
+			oob_data = NDIS_PACKET_OOB_DATA(packet);
+			oob_data->status = NDIS_STATUS_NOT_RECOGNIZED;
+			if (!(wnd->attributes & NDIS_ATTRIBUTE_DESERIALIZE))
+				irql = raise_irql(DISPATCH_LEVEL);
 			res = LIN2WIN3(miniport->send, wnd->nmb->adapter_ctx,
 				       packet, packet->private.flags);
-			if (res == NDIS_STATUS_SUCCESS) {
+			if (!(wnd->attributes & NDIS_ATTRIBUTE_DESERIALIZE))
+				lower_irql(irql);
+			switch (res) {
+			case NDIS_STATUS_SUCCESS:
 				free_tx_packet(wnd, packet, res);
-				continue;
-			} else {
-				DBGTRACE3("%08X", res);
+				break;
+			case NDIS_STATUS_PENDING:
+				break;
+			case NDIS_STATUS_RESOURCES:
+				wnd->tx_ok = 0;
+				/* resend this packet when resources
+				 * become available */
 				i--;
-				if (res == NDIS_STATUS_RESOURCES)
-					wnd->tx_ok = 0;
+				break;
+			case NDIS_STATUS_FAILURE:
+				free_tx_packet(wnd, packet, res);
+				break;
+			default:
+				ERROR("packet %p: invalid status: %08X",
+				      packet, res);
 				break;
 			}
 		}
-		if (!(wnd->attributes & NDIS_ATTRIBUTE_DESERIALIZE))
-			lower_irql(irql);
 		sent = i + 1;
 	}
 	TRACEEXIT3(return sent);
@@ -760,7 +770,10 @@ static void ndis_poll_controller(struct net_device *dev)
 	struct wrap_ndis_device *wnd = netdev_priv(dev);
 
 	disable_irq(dev->irq);
-	ndis_isr(dev->irq, wnd, NULL);
+	if (wnd->ndis_irq->req_isr)
+		ndis_isr_shared(dev->irq, wnd, NULL);
+	else
+		ndis_isr_dynamic(dev->irq, wnd, NULL);
 	enable_irq(dev->irq);
 }
 #endif
